// config/passport.js

// load all the things we need
const LocalStrategy   = require('passport-local').Strategy;

// load up the user model
const User            = require('../models/user');

// expose this function to our app using module.exports
module.exports = function(passport) {

    // =========================================================================
    // passport session setup ==================================================
    // =========================================================================
    // required for persistent login sessions
    // passport needs ability to serialize and unserialize users out of session

    // used to serialize the user for the session
    passport.serializeUser(function(user, done) {
        done(null, user.id);
    });

    // used to deserialize the user
    passport.deserializeUser(function(id, done) {
        User.findById(id, function(err, user) {
            done(err, user);
        });
    });

    // =========================================================================
    // LOCAL SIGNUP ============================================================
    // =========================================================================
    // we are using named strategies since we have one for login and one for signup
    // by default, if there was no name, it would just be called 'local'

    passport.use('local-signup', new LocalStrategy({
        // by default, local strategy uses username and password, we will override with pseudo
        usernameField : 'pseudo',
        passwordField : 'password',
        passReqToCallback : true // allows us to pass back the entire request to the callback
    },
    function(req, pseudo, password, done) {

        // asynchronous
        // User.findOne wont fire unless data is sent back
        process.nextTick(function() {

            // find a user whose pseudo is the same as the forms email
            // we are checking to see if the user trying to login already exists
            User.findOne({ 'local.pseudo' :  pseudo }, function(err, user) {
                // if there are any errors, return the error
                if (err)
                    return done(err);

                // check to see if theres already a user with that email
                if (user) {
                    return done(null, false, req.flash('signupMessage', 'Ce pseudo est déjà pris. Sorry! =/'));
                } else {

                    // if there is no user with that pseudo
                    // create the user
                    const newUser            = new User();

                    // set the user's local credentials
                    newUser.local.pseudo   = pseudo;
                    newUser.local.password = newUser.generateHash(password);

                    // save the user
                    newUser.save(function(err) {
                        if (err)
                            throw err;
                        return done(null, newUser);
                    });
                }
            });
        });
    }));

    // =========================================================================
    // LOCAL LOGIN =============================================================
    // =========================================================================
    // we are using named strategies since we have one for login and one for signup
    // by default, if there was no name, it would just be called 'local'

    passport.use('local-login', new LocalStrategy({
        // by default, local strategy uses username and password, we could override with email (see before)
        usernameField : 'pseudo',
        passwordField : 'password',
        passReqToCallback : true // allows us to pass back the entire request to the callback
    },
    function(req, pseudo, password, done) { // callback with email and password from our form

        // find a user whose email is the same as the forms email
        // we are checking to see if the user trying to login already exists

        User.findOne({ 'local.pseudo' :  pseudo }, function(err, user) {

            // if there are any errors, return the error before anything else
            if (err) {
                return done(err);
            }

            // if no user is found, return the message
            if (!user) {                
                return done(null, false,
                    req.flash('loginMessage',
                    "Je n'ai pas souvenir de t'avoir rencontré. Tu es sûr de ton login ?")); // req.flash is the way to set flashdata using connect-flash
            }

            // if the user is found but the password is wrong
            if (!user.validPassword(password)) {
                return done(null, false,
                    req.flash('loginMessage', 'Oops! Tu es sûr du mot de passe ?'));
            }

            // all is well, return successful user
            return done(null, user);

        });
    }));
};